package types

import (
	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
)

const TypeMsgClaimFunds = "claim_funds"

var _ sdk.Msg = &MsgClaimFunds{}

func NewMsgClaimFunds(creator string, auctionId string) *MsgClaimFunds {
	return &MsgClaimFunds{
		Creator:   creator,
		AuctionId: auctionId,
	}
}

func (msg *MsgClaimFunds) Route() string {
	return RouterKey
}

func (msg *MsgClaimFunds) Type() string {
	return TypeMsgClaimFunds
}

func (msg *MsgClaimFunds) GetSigners() []sdk.AccAddress {
	creator, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		panic(err)
	}
	return []sdk.AccAddress{creator}
}

func (msg *MsgClaimFunds) GetSignBytes() []byte {
	bz := ModuleCdc.MustMarshalJSON(msg)
	return sdk.MustSortJSON(bz)
}

func (msg *MsgClaimFunds) ValidateBasic() error {
	_, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return sdkerrors.Wrapf(sdkerrors.ErrInvalidAddress, "invalid creator address (%s)", err)
	}
	return nil
}
